# Copyright (C) 2025 Embedl AB

{% set H = IMG_H | default(224, true) %}
{% set W = IMG_W | default(224, true) %}

img_size: &IMG_SIZE [{{ H }}, {{ W }}]
batch_size: 'auto'
pre_trained: True
weight_decay: 0.01
max_learning_rate: 'auto'
train_transforms:
  - {type: Resize, size: *IMG_SIZE}
  - {type: ToTensor}
val_transforms:
  - {type: Resize, size: *IMG_SIZE}
  - {type: ToTensor}
